// Required functions
// ==================

// void main()
//   Entry point for the application. Returning from main will reset the NES.

// void nmi_sound()
//   Called from NMI
//   You may wish to implement it in assembly as an _nmi_sound: subroutine.

// void irq()
//   Called from IRQ.
//   If implementing IRQs, you will likely want to directly change vector_irq
//   to point to an assembly coded routine instead.

// General
// =======

typedef unsigned char      uint8;
typedef signed char        sint8;
typedef unsigned short int uint16;
typedef signed short int   sint16;

#define NULL 0

// RAW_BUS: for advanced users, allows peek/poke at any memory address.
#define RAW_BUS(x) (*((volatile uint8*)x))

void exit(); // resets the NES

// generic static variables to avoid stack
extern uint8  i, j;
extern uint16 k, l;
extern uint8  a, b;

#pragma zpsym("i");
#pragma zpsym("j");
#pragma zpsym("k");
#pragma zpsym("l");
#pragma zpsym("a");
#pragma zpsym("b");

// Gamepad
// =======

extern uint8 gamepad0;
extern uint8 gamepad1;

#pragma zpsym("gamepad0")
#pragma zpsym("gamepad1")

// gamepad state mask
#define PAD_A        0x01
#define PAD_B        0x02
#define PAD_SELECT   0x04
#define PAD_START    0x08
#define PAD_U        0x10
#define PAD_D        0x20
#define PAD_L        0x40
#define PAD_R        0x80

// gamepad_poll: updates gamepad0/1 with current gamepad state
//               This does not support Famicom expansion port controllers.
//               This is not safe for use when DPCM is playing.
void gamepad_poll();

// PPU interaction
// ===============

// ppu_oam: 256 byte sprite buffer uploaded to PPU OAM during vblank
extern uint8 ppu_oam[256];

// ppu_nam: nametable command buffer, delivered to PPU during vblank
//   buffer format:
//     a, b, c, ...
//     a - a & $7F is bytes to write, 0 is end of buffer
//         a & $80 selects horizontal/vertical writes (0/1)
//     b - high PPU address
//     c - low PPU address
//     ... - bytes to be written
//extern uint8 ppu_nam[256];
extern uint8 ppu_nam[64]; // made smaller to conserve RAM

// ppu_pal: palette buffer, delivered to PPU during vblank
extern uint8 ppu_pal[32];

// ppu_send: set this to true when ppu_oam/ppu_nmt/ppu_pal are filled
//           and ready to be sent duing next vblank.
extern uint8 ppu_send;
#pragma zpsym("ppu_send")

// ppu_frame: incremented once at the end of each NMI
extern uint8 ppu_frame;
#pragma zpsym("ppu_frame")

// ppu_ctrl: state of ppu_ctrl ($2000 register), updated during NMI. bit 7 is ignored
extern uint8 ppu_ctrl;  // initial value: %00001000
#pragma zpsym("ppu_ctrl")

// ppu_mask: state of ppu_mask ($2001 register), updated during NMI.
extern uint8 ppu_mask;  // initial value: %00011110
#pragma zpsym("ppu_mask")

// ppu_scroll_x/y: screen scroll position, updated during NMI
extern uint8 ppu_scroll_x;
extern uint8 ppu_scroll_y;
#pragma zpsym("ppu_scroll_x")
#pragma zpsym("ppu_scroll_y")

// ppu_render_off: turns off rendering, making it safe to interact with PPU memory directly.
//                 Setting ppu_send will turn rendering back on during the next NMI.
//                 If ppu_send might be true, it is safest to call nmi_wait() to clear it
//                 before disabling rendering.
void ppu_render_off();

// ppu_nmi_off/on: controls the NMI interrupt from the PPU
void ppu_nmi_off();
void ppu_nmi_on();

// ppu_nmi_wait: waits for the next NMI to complete.
//               After setting ppu_send to nonzero, call this function
void ppu_nmi_wait();

// ppu_sprite_0_hit: waits until sprite 0 hit flag is set
void ppu_sprite_0_hit();

// ppu_latch_h/v: Latch the PPU write address for ppu_write (see below).
//                Sets either horizontal or vertical increment for writing.
//                Rendering and NMI should be off before using this.
void __fastcall__ ppu_latch_h(uint16 addr);
void __fastcall__ ppu_latch_v(uint16 addr);

// ppu_write: writes a byte to the PPU after latching (see above).
//            Increments the PPU write address after each write.
//            In vertical latch mode, the address is incremented by 32 after each write.
void __fastcall__ ppu_write(uint8 data);

// debug_grey: immediately sets the greyscale bit in ppu_ctrl ($2001 register).
//             Useful for visual code timing (grey area shows time left in frame).
//             Make sure to set ppu_send = 1 to reset the greyscale during NMI.
void debug_grey();

// TODO
// ====

// APU interaction
// DPCM safe controller reading, Famicom expansion controller reading.

// end of file